/* eslint-disable dot-notation,no-param-reassign,no-underscore-dangle */

import { makePath } from './helper';

const Empty = ({ children }) => {
  return children || null;
};

const parsePageToRoute = (page, componentMapper) => {
  const { pages, childRoutes, component, entry, indexRoute, ...route } = page;
  if (!component) {
    route.component = Empty;
  } else {
    const comp = componentMapper[component];
    if (!comp) {
      throw new Error(`Invalid component: ${component}.`);
    }
    route.component = comp;
  }
  const thePages = pages || childRoutes;
  if (thePages && thePages.length > 0) {
    route.childRoutes = thePages.map(p => parsePageToRoute(p, componentMapper));
  }
  const theIndexRoute = entry || indexRoute;
  if (theIndexRoute) {
    if (typeof theIndexRoute !== 'object') {
      throw new Error('The \'entry\' must be an object with struct: { path: string, dsName: string, params: object }.');
    }
    const { path: entryPath, ...state } = theIndexRoute;
    if (!entryPath || findRoutesByPath(route.childRoutes, entryPath).length === 0) {
      throw new Error('The \'entry\' should have a valid path property.');
    }
    route.indexRoute = {
      onEnter(nextState, replace) {
        const theState = nextState.state || {};
        replace(makePath(nextState.match.url, entryPath), { ...theState, ...state });
      },
    };
  }
  return route;
};

const dealWithPath = (path) => {
  if (!path || path === '/') {
    return [];
  }
  let noStart = false;
  let noEnd = false;
  if (path.startsWith('/')) {
    noStart = true;
  }
  if (path.endsWith('/')) {
    noEnd = true;
  }
  if (noStart || noEnd) {
    path = path.substring(noStart ? 1 : 0, noEnd ? (path.length - 1) : path.length);
  }
  return path.split('/');
};

const findRoutesByPath = (routes, path) => {
  const parts = dealWithPath(path);
  return _findRoutesByPath(routes, parts);
};

const _findRouteByPath = (route, parts) => {
  if (!route || parts.length === 0) {
    return [];
  }
  const [current, ...others] = parts;
  if (route.path === current) {
    if (parts.length > 1) {
      if (route.childRoutes) {
        const res = _findRoutesByPath(route.childRoutes, others);
        if (res.length > 0) {
          return [route, ...res];
        } else {
          return [];
        }
      } else {
        return [];
      }
    } else {
      return [route];
    }
  }
};

const _findRoutesByPath = (routes, parts) => {
  if (!routes) {
    return [];
  }
  for (const route of routes) {
    const res = _findRouteByPath(route, parts);
    if (res.length > 0) {
      return res;
    }
  }
  return [];
};

export const parseLayout = (layout, componentMapper) => {
  const route = {};
  if (layout['pages'] || layout.childRoutes) {
    route.childRoutes = (layout['pages'] || layout.childRoutes).map(page => parsePageToRoute(page, componentMapper));
  } else {
    throw new Error('No pages is found!');
  }
  if (layout.entry) {
    if (typeof layout.entry !== 'object') {
      throw new Error('The \'entry\' must be an object with struct: { path: string, dsName: string, params: object }.');
    }
    const { path: entryPath, ...state } = layout.entry;
    if (!entryPath || findRoutesByPath(route.childRoutes, entryPath).length === 0) {
      throw new Error('The \'entry\' should have a valid path property.');
    }
    route.indexRoute = {
      onEnter(nextState, replace) {
        replace(makePath(nextState.match.url, entryPath), state);
      },
    };
  } else {
    throw new Error('No entry is found!');
  }
  return route;
};
